/* **********************************************************
 * Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
 * **********************************************************/


/*
 * iocontrols.h
 *
 *        The driver io controls.
 */

#ifndef _IOCONTROLS_H_
#define _IOCONTROLS_H_

#define INCLUDE_ALLOW_USERLEVEL
#define INCLUDE_ALLOW_VMMEXT
#define INCLUDE_ALLOW_MODULE
#define INCLUDE_ALLOW_VMKERNEL
#define INCLUDE_ALLOW_VMCORE
#include "includeCheck.h"

#include "pshare_ext.h"

/*
 * Driver version.
 *
 * Increment major version when you make an incompatible change.
 * Compatibility goes both ways (old driver with new executable
 * as well as new driver with old executable).
 */

#define VMMON_VERSION_GSX1	(3 << 16 | 0)
#define VMMON_VERSION_V3	(8 << 16 | 0)
#define VMMON_VERSION_V32	(10 << 16 | 0)
#define VMMON_VERSION_GSX2	(21 << 16 | 0)
#define VMMON_VERSION_V321	(30 << 16 | 0)
#define VMMON_VERSION_GSX25	(41 << 16 | 0)
#define VMMON_VERSION_GSX251	(42 << 16 | 0)
#define VMMON_VERSION_V4	(52 << 16 | 0)
#define VMMON_VERSION_V45	(79 << 16 | 0)
#define VMMON_VERSION_V452	(81 << 16 | 0)
#define VMMON_VERSION_TOT	(100 << 16 | 0)
#define VMMON_VERSION_MAJOR(v)	((uint32) (v) >> 16)
#define VMMON_VERSION_MINOR(v)	((uint16) (v))


/*
 * ENOMEM returned after MAX_VMS virtual machines created
 */

#ifdef VMX86_SERVER
#define MAX_VMS 80
#else
#define MAX_VMS 24
#endif


/*
 * We need to specify a special base for ioctl command.
 *
 * This is because in x86_64 Linux, we need to explicitly register
 * handlers for ioctl command numbers with the host kernel to handle
 * 32 bit ioctl system calls, which is a stupid thing done by the
 * kernel folks. In i386 Linux, such a requirement is not necessary.
 * 
 * A cleaner interface would be to define only one ioctl cmd to be
 * used by all calls into vmmon and pass in our customized commands
 * through a parameter. The advantage is that it's more flexible
 * for use to work around Linux kernel hack like in x86_64. The
 * disadvantage is that we need to translate user address to kernel
 * address for all ioctl calls.
 */

typedef enum IOCTLCmd {
   IOCTLCMD_GET_MHZ_ESTIMATE = 1,
   IOCTLCMD_NOT_IMPLEMENTED,

   IOCTLCMD_NONE = 2000,
   IOCTLCMD_VERSION,
   IOCTLCMD_CREATE_VM,
   IOCTLCMD_BIND_VM,
   IOCTLCMD_RELEASE_VM,
   IOCTLCMD_GET_NUM_VMS,
   IOCTLCMD_INIT_VM,
   IOCTLCMD_LATE_INIT_VM,
   IOCTLCMD_RUN_VM,
   IOCTLCMD_LOOK_UP_MPN,
   IOCTLCMD_LOCK_PAGE,
   IOCTLCMD_UNLOCK_PAGE,
   IOCTLCMD_APIC_BASE,
   IOCTLCMD_GET_HARD_LIMIT,
   IOCTLCMD_SET_HARD_LIMIT,
   IOCTLCMD_GET_MEM_INFO,
   IOCTLCMD_ADMIT,
   IOCTLCMD_SET_MEM_USAGE,
   IOCTLCMD_READMIT,
   IOCTLCMD_PAE_ENABLED,
   IOCTLCMD_HOST_X86_64,
   IOCTLCMD_GET_TOTAL_MEM_USAGE,
   IOCTLCMD_ACK_USER_CALL,
   IOCTLCMD_COMPLETE_USER_CALL,
   IOCTLCMD_GET_KHZ_ESTIMATE,
   IOCTLCMD_SET_HOST_CLOCK_RATE,
   IOCTLCMD_READ_PAGE,
   IOCTLCMD_WRITE_PAGE,
   IOCTLCMD_LOCK_PAGE_NEW,
   IOCTLCMD_UNLOCK_PAGE_BY_MPN,
   IOCTLCMD_MARK_LOCKEDVARANGE_CLEAN,
   IOCTLCMD_COW_SHARE,
   IOCTLCMD_COW_CHECK,
   IOCTLCMD_COW_UPDATE_HINT,
   IOCTLCMD_COW_COPY_PAGE,
   IOCTLCMD_COW_REMOVE_HINT,
    /* AWE calls */
   IOCTLCMD_ALLOC_LOCKED_PAGES,
   IOCTLCMD_FREE_LOCKED_PAGES,
   IOCTLCMD_GET_LOCKED_PAGES_LIST,
   IOCTLCMD_MAP_LOCKED_PAGES,
   IOCTLCMD_UNMAP_LOCKED_PAGES,

   /*
    * Keep host-specific calls at the end so they can be undefined
    * without renumbering the common calls.
    */

#ifdef linux
   IOCTLCMD_SET_UID,		// VMX86_DEVEL only
   IOCTLCMD_IS_MP_SAFE,
   IOCTLCMD_ALLOW_CORE_DUMP,

   IOCTLCMD_BROADCAST_IPI,	// SMP 2.2.8+ only

   IOCTLCMD_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_FREE_PASSTHROUGH_IO,
   IOCTLCMD_FREE_PASSTHROUGH_IRQ,
   IOCTLCMD_START_PASSTHROUGH,
   IOCTLCMD_STOP_PASSTHROUGH,
   IOCTLCMD_QUERY_PASSTHROUGH,

   IOCTLCMD_REGISTER_PERFCTR,
   IOCTLCMD_START_PERFCTR,
   IOCTLCMD_STOP_PERFCTR,
   IOCTLCMD_RELEASE_PERFCTR,

   IOCTLCMD_ALLOC_LOW_PAGES,
   IOCTLCMD_FREE_LOW_PAGES,

   IOCTLCMD_GET_ALL_CPUID,

   IOCTLCMD_VMFS_GET_FREE_SPACE = 3000,
   IOCTLCMD_VMFS_PHYS_MEM_IO,
#endif

#ifdef _WIN32
   IOCTLCMD_BEEP,
   IOCTLCMD_HARD_LIMIT_MONITOR_STATUS,	// Windows 2000 only
   IOCTLCMD_BLUE_SCREEN,	// USE_BLUE_SCREEN only
   IOCTLCMD_CHANGE_HARD_LIMIT,
   IOCTLCMD_ALLOC_LOW_PAGES,
   IOCTLCMD_FREE_LOW_PAGES,
   IOCTLCMD_SYNC_GET_TSCS,
   IOCTLCMD_SYNC_SET_TSCS,
   IOCTLCMD_SEND_IPI,
#endif

#ifdef VMX86_SERVER
   /* 
    * Used to open / close vmfs files (to ensure that the files
    * are properly closed when the process goes away)
    */
   IOCTLCMD_FILE_OPEN,
   IOCTLCMD_FILE_CLOSE,
#endif

   IOCTLCMD_LAST
} IOCTLCmd;


/*
 * Windows ioctl definitions.
 *
 * We use the IRP Information field for the return value
 * of IOCTLCMD_RUN, to be faster since it is use a lot.
 */

#ifdef _WIN32

#define FILE_DEVICE_VMX86        0x8101
#define VMX86_IOCTL_BASE_INDEX   0x801
#define VMIOCTL_BUFFERED(name) \
      CTL_CODE(FILE_DEVICE_VMX86, \
	       VMX86_IOCTL_BASE_INDEX+IOCTLCMD_##name, \
	       METHOD_BUFFERED, \
	       FILE_ANY_ACCESS)
#define VMIOCTL_NEITHER(name) \
      CTL_CODE(FILE_DEVICE_VMX86, \
	       VMX86_IOCTL_BASE_INDEX+IOCTLCMD_##name, \
	       METHOD_NEITHER, \
	       FILE_ANY_ACCESS)

#define IOCTL_VMX86_VERSION		VMIOCTL_BUFFERED(VERSION)
#define IOCTL_VMX86_CREATE_VM		VMIOCTL_BUFFERED(CREATE_VM)
#define IOCTL_VMX86_BIND_VM		VMIOCTL_BUFFERED(BIND_VM)
#define IOCTL_VMX86_RELEASE_VM		VMIOCTL_BUFFERED(RELEASE_VM)
#define IOCTL_VMX86_GET_NUM_VMS		VMIOCTL_BUFFERED(GET_NUM_VMS)
#define IOCTL_VMX86_INIT_VM		VMIOCTL_BUFFERED(INIT_VM)
#define IOCTL_VMX86_LATE_INIT_VM	VMIOCTL_BUFFERED(LATE_INIT_VM)
#define IOCTL_VMX86_RUN_VM		VMIOCTL_NEITHER(RUN_VM)
#define IOCTL_VMX86_SEND_IPI		VMIOCTL_NEITHER(SEND_IPI)
#define IOCTL_VMX86_LOOK_UP_MPN		VMIOCTL_BUFFERED(LOOK_UP_MPN)
#define IOCTL_VMX86_LOCK_PAGE		VMIOCTL_BUFFERED(LOCK_PAGE)
#define IOCTL_VMX86_UNLOCK_PAGE		VMIOCTL_BUFFERED(UNLOCK_PAGE)
#define IOCTL_VMX86_APIC_BASE		VMIOCTL_BUFFERED(APIC_BASE)
#define IOCTL_VMX86_GET_HARD_LIMIT	VMIOCTL_BUFFERED(GET_HARD_LIMIT)
#define IOCTL_VMX86_SET_HARD_LIMIT	VMIOCTL_BUFFERED(SET_HARD_LIMIT)
#define IOCTL_VMX86_GET_MEM_INFO	VMIOCTL_BUFFERED(GET_MEM_INFO)
#define IOCTL_VMX86_ADMIT  	        VMIOCTL_BUFFERED(ADMIT)
#define IOCTL_VMX86_READMIT             VMIOCTL_BUFFERED(READMIT)
#define IOCTL_VMX86_SET_MEM_USAGE       VMIOCTL_BUFFERED(SET_MEM_USAGE)
#define IOCTL_VMX86_PAE_ENABLED		VMIOCTL_BUFFERED(PAE_ENABLED)
#define IOCTL_VMX86_HOST_X86_64         VMIOCTL_BUFFERED(HOST_X86_64)
#define IOCTL_VMX86_COW_SHARE           VMIOCTL_BUFFERED(COW_SHARE)
#define IOCTL_VMX86_COW_CHECK           VMIOCTL_BUFFERED(COW_CHECK)
#define IOCTL_VMX86_COW_UPDATE_HINT     VMIOCTL_BUFFERED(COW_UPDATE_HINT)
#define IOCTL_VMX86_COW_COPY_PAGE       VMIOCTL_BUFFERED(COW_COPY_PAGE)
#define IOCTL_VMX86_COW_REMOVE_HINT     VMIOCTL_BUFFERED(COW_REMOVE_HINT)

#define IOCTL_VMX86_BEEP		VMIOCTL_BUFFERED(BEEP)
#define IOCTL_VMX86_HARD_LIMIT_MONITOR_STATUS	VMIOCTL_BUFFERED(HARD_LIMIT_MONITOR_STATUS)
#define IOCTL_VMX86_CHANGE_HARD_LIMIT   VMIOCTL_BUFFERED(CHANGE_HARD_LIMIT)
#define IOCTL_VMX86_ALLOC_LOW_PAGES     VMIOCTL_BUFFERED(ALLOC_LOW_PAGES)
#define IOCTL_VMX86_FREE_LOW_PAGES      VMIOCTL_BUFFERED(FREE_LOW_PAGES)
#ifdef USE_BLUE_SCREEN
#define IOCTL_VMX86_BLUE_SCREEN		VMIOCTL_BUFFERED(BLUE_SCREEN)
#endif

#define IOCTL_VMX86_GET_VM_LIST		VMIOCTL_BUFFERED(GET_VM_LIST)
#define IOCTL_VMX86_GET_VM_INFO		VMIOCTL_BUFFERED(GET_VM_INFO)
#define IOCTL_VMX86_SET_VM_INFO		VMIOCTL_BUFFERED(SET_VM_INFO)
#define IOCTL_VMX86_GET_TOTAL_MEM_USAGE	VMIOCTL_BUFFERED(GET_TOTAL_MEM_USAGE)
#define IOCTL_VMX86_ACK_USER_CALL       VMIOCTL_BUFFERED(ACK_USER_CALL)
#define IOCTL_VMX86_COMPLETE_USER_CALL  VMIOCTL_BUFFERED(COMPLETE_USER_CALL)
#define IOCTL_VMX86_GET_KHZ_ESTIMATE    VMIOCTL_BUFFERED(GET_KHZ_ESTIMATE)
#define IOCTL_VMX86_SET_HOST_CLOCK_RATE VMIOCTL_BUFFERED(SET_HOST_CLOCK_RATE)
#define IOCTL_VMX86_SYNC_GET_TSCS       VMIOCTL_BUFFERED(SYNC_GET_TSCS)
#define IOCTL_VMX86_SYNC_SET_TSCS       VMIOCTL_BUFFERED(SYNC_SET_TSCS)
#define IOCTL_VMX86_READ_PAGE           VMIOCTL_BUFFERED(READ_PAGE)
#define IOCTL_VMX86_WRITE_PAGE          VMIOCTL_BUFFERED(WRITE_PAGE)
#define IOCTL_VMX86_LOCK_PAGE_NEW	VMIOCTL_BUFFERED(LOCK_PAGE_NEW)
#define IOCTL_VMX86_UNLOCK_PAGE_BY_MPN  VMIOCTL_BUFFERED(UNLOCK_PAGE_BY_MPN)
#define IOCTL_VMX86_ALLOC_LOCKED_PAGES	VMIOCTL_BUFFERED(ALLOC_LOCKED_PAGES)
#define IOCTL_VMX86_FREE_LOCKED_PAGES   VMIOCTL_BUFFERED(FREE_LOCKED_PAGES)
#define IOCTL_VMX86_GET_LOCKED_PAGES_LIST VMIOCTL_BUFFERED(GET_LOCKED_PAGES_LIST)
#define IOCTL_VMX86_MAP_LOCKED_PAGES    VMIOCTL_BUFFERED(MAP_LOCKED_PAGES)
#define IOCTL_VMX86_UNMAP_LOCKED_PAGES  VMIOCTL_BUFFERED(UNMAP_LOCKED_PAGES)

#endif


/*
 * Limit structure for CREATE_VM
 */

#define WSLIMITS_HASH_SIZE 20
typedef struct WSLimitsInfo {
   int32   memsize;   /* Memory size of this VM in MB */
   char    hash[WSLIMITS_HASH_SIZE];
} WSLimitsInfo;



/*
 * Return codes from page locking, unlocking, and MPN lookup.
 * They share an error code space because they call one another
 * internally.
 *
 *    PAGE_LOCK_FAILED              The host refused to lock a page.
 *    PAGE_LOCK_LIMIT_EXCEEDED      We have reached the limit of locked
 *                                  pages for all VMs
 *    PAGE_LOCK_TOUCH_FAILED        Failed to touch page after lock.
 *    PAGE_LOCK_IN_TRANSITION       The page is locked but marked by Windows
 *                                  as nonpresent in CPU PTE and in transition 
 *                                  in Windows PFN.  
 *
 *    PAGE_LOCK_SYS_ERROR           System call error.
 *    PAGE_LOCK_ALREADY_LOCKED      Page already locked.
 *    PAGE_LOCK_MEMTRACKER_ERROR    MemTracker fails.
 *    PAGE_LOCK_PHYSTRACKER_ERROR   PhysTracker fails.
 *    PAGE_LOCK_MDL_ERROR           Mdl error on Windows.
 *
 *    PAGE_UNLOCK_NO_ERROR          Unlock successful (must be 0).
 *    PAGE_UNLOCK_NOT_TRACKED       Not in memtracker.
 *    PAGE_UNLOCK_NO_MPN            Tracked but no MPN.
 *    PAGE_UNLOCK_NOT_LOCKED        Not locked.
 *    PAGE_UNLOCK_TOUCH_FAILED      Failed to touch page.
 *    PAGE_UNLOCK_MISMATCHED_TYPE   Tracked but was locked by different API 
 *
 *    PAGE_LOOKUP_INVALID_ADDR      Consistency checking on Windows.
 *    PAGE_LOOKUP_BAD_HIGH_ADDR     Consistency checking on Windows.
 *    PAGE_LOOKUP_ZERO_ADDR         Consistency checking on Windows.
 *    PAGE_LOOKUP_SMALL_ADDR        Consistency checking on Windows.
 *
 * PAGE_LOCK_ERROR_BIT is used to encode the error value as a positive
 * number to get through ioctl().
 *
 * -- edward
 */

#define PAGE_LOCK_FAILED                  -1
#define PAGE_LOCK_LIMIT_EXCEEDED          -2
#define PAGE_LOCK_TOUCH_FAILED            -3
#define PAGE_LOCK_IN_TRANSITION           -4

#define PAGE_LOCK_SYS_ERROR              -10
#define PAGE_LOCK_ALREADY_LOCKED         -11
#define PAGE_LOCK_MEMTRACKER_ERROR       -12
#define PAGE_LOCK_PHYSTRACKER_ERROR      -13
#define PAGE_LOCK_MDL_ERROR              -14

#define PAGE_UNLOCK_NO_ERROR		   0
#define PAGE_UNLOCK_NOT_TRACKED		-100
#define PAGE_UNLOCK_NO_MPN		-101
#define PAGE_UNLOCK_NOT_LOCKED		-102
#define PAGE_UNLOCK_TOUCH_FAILED	-103
#define PAGE_UNLOCK_MISMATCHED_TYPE	-104

#define PAGE_LOOKUP_INVALID_ADDR        -200
#define PAGE_LOOKUP_BAD_HIGH_ADDR       -201
#define PAGE_LOOKUP_ZERO_ADDR           -202
#define PAGE_LOOKUP_SMALL_ADDR          -203
#define PAGE_LOOKUP_NOT_TRACKED          -10	// added to another code
#define PAGE_LOOKUP_NO_MPN               -20	// added to another code
#define PAGE_LOOKUP_NOT_LOCKED           -30	// added to another code
#define PAGE_LOOKUP_NO_VM                -40	// added to another code

#define PAGE_LOCK_SUCCESS(mpn) ((int) (mpn) >= 0)
#define PAGE_LOCK_SOFT_FAILURE(mpn) ((int) (mpn) < 0 && (int) (mpn) > -10)

#define PAGE_LOCK_ERROR_BIT 0x4000000


/*
 * Flags sent into APICBASE ioctl
 */

#define APIC_FLAG_DISABLE_NMI       0x00000001
#define APIC_FLAG_PROBE             0x00000002
#define APIC_FLAG_FORCE_ENABLE      0x00000004


/*
 * Structure sent into REGISTER_PERFCOUNTERS ioctl
 */

#define PERFCTR_MAX_COUNTERS        2
#define PERFCTR_INVALID_EVENT       0
#define PERFCTR_INVALID_IRQ         -1

typedef struct PerfCtrInfo {
   uint32 eventNum;
   uint32 samplingRate;
} PerfCtrInfo;

typedef struct PerfCtrRegisterArgs {
   PerfCtrInfo counterInfo[PERFCTR_MAX_COUNTERS];
   int irq;
   Bool totalOnly;
} PerfCtrRegisterArgs;

/* Structure to get VMFS free space */
typedef struct VMFSFreeSpaceArgs {
   int64 bytesFree;
} VMFSFreeSpaceArgs;

typedef struct VMAPICInfo {
   uint32 flags;
   MA32   apicBase;
} VMAPICInfo; 

#ifndef MAX_PROCESSORS
#define MAX_PROCESSORS 32
#endif

typedef struct VMSyncWriteTSCs {
   uint32 cpuSet;
   uint32 TSCs[MAX_PROCESSORS];
} VMSyncWriteTSCs;


#define VMX86_DRIVER_VCPUID_OFFSET	1000

// Magic file system number for VMFS
#define FS_MAGIC_NUMBER		0x2fabf15e


/*
 * We keep track of 3 different limits on the number of pages we can lock.
 * The host limit is determined at driver load time (in windows only) to
 * make sure we do not starve the host by locking too many pages.
 * The static limit is user defined in the UI and the dynamic limit is 
 * set by authd's hardLimitMonitor code (windows only), which queries
 * host load and adjusts the limit accordingly.  We lock the minimum of 
 * all these values at any given time.
 */
typedef struct LockedPageLimit {
   uint32 host;        // driver calculated maximum for this host
   uint32 configured;  // user defined maximum pages to lock
   uint32 dynamic;     // authd hardLimitMonitor pages to lock
} LockedPageLimit;

/*
 * Data structures for the GET_MEM_INFO and ADMIT ioctls.
 */

typedef struct VMMemMgmtInfo {
   uint32          minAllocation;   // minimum pages for vm
   uint32          maxAllocation;   // maximum pages the vm could lock
   uint32          shares;          // proportional sharing weight
   uint32          nonpaged;        // overhead memory
   uint32          paged;           // nonmainMem guest memory
   uint32          locked;          // number of pages locked by this vm
   uint32          usedPct;         // % of guest memory being touched
   uint32          admitted;        // admission control
} VMMemMgmtInfo;

#define VMMEM_COW_HOT_PAGES 5

typedef struct VMMemCOWHotPage {
   uint32          mpn;
   uint32          ref;
   uint64          key;
} VMMemCOWHotPage;

typedef struct VMMemCOWInfo {
   uint32          numHints;
   uint32          numPages;
   uint32          numBreaks;
   uint32          numRef;
   VMMemCOWHotPage hot[VMMEM_COW_HOT_PAGES];
} VMMemCOWInfo;

typedef struct VMMemInfoArgs {
   VMMemCOWInfo    cowInfo;            // global page sharing state
   uint32          numVMs;             
   uint32          minVmMemPct;        // % of vm that must fit in maxLockedPages
   uint32          globalMinAllocation;// pages that must fit in maxLockedPages
   uint32          numLockedPages;     // total locked pages by all vms
   LockedPageLimit lockedPageLimit;    // set of locked page limits
   uint32          maxLockedPages;     // effective limit on locked pages
   uint32          callerIndex;        // this vm's index memInfo array
   VMMemMgmtInfo   memInfo[1];
   uint8           _pad[4];
} VMMemInfoArgs;

#define VM_GET_MEM_INFO_SIZE(numVMs) \
   ((numVMs - 1) * sizeof (VMMemMgmtInfo) + sizeof (VMMemInfoArgs))

/*
 * Structure for GET_STATS and SET_STATS.
 */

typedef struct {
  uint32 userLockedPages;	/* current count of pages locked by app */
  uint32 maxUserLockedPages;	/* max over time of pages locked by app */
  uint32 monitorLockedPages;	/* current count of pages locked by monitor */
  uint32 maxMonitorLockedPages;	/* max over time of pages locked by monitor */
} VMX86StatsV4;

typedef struct VMGetPCIInfoArgs {
   int busNumber;
   int deviceNumber;
   int functionNumber;
} VMGetPCIInfoArgs;

typedef struct VMGetPCIInfoResult {
   int bus;
   int slot;
   int vendorID;
   int deviceID;
   int baseClass;
   int subClass;
   int interruptLine;
   int interruptVector;
} VMGetPCIInfoResult;

typedef enum IOCTLCmdV2 { 
   IOCTLCMD_V2_NONE=200,
   IOCTLCMD_V2_INIT,
   IOCTLCMD_V2_LATE_INIT,
   IOCTLCMD_V2_RUN,
   IOCTLCMD_V2_LOOKUPMPN,
   IOCTLCMD_V2_BEEP,   // 205 
   IOCTLCMD_V2_SETUID, // Linux and VMX86_DEVEL only
   IOCTLCMD_V2_LOCKPAGE, 
   IOCTLCMD_V2_UNLOCKPAGE, 
   IOCTLCMD_V2_ISMPSAFE,
   IOCTLCMD_V2_APICBASE, // 210
   IOCTLCMD_V2_IOAPICBASE,
   IOCTLCMD_V2_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_V2_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_V2_FREE_PASSTHROUGH_IO,
   IOCTLCMD_V2_FREE_PASSTHROUGH_IRQ, // 215
   IOCTLCMD_V2_START_PASSTHROUGH,
   IOCTLCMD_V2_STOP_PASSTHROUGH,
   IOCTLCMD_V2_QUERY_PASSTHROUGH,
   IOCTLCMD_V2_GET_NUM_VMS,
   IOCTLCMD_V2_DECLARE_SLAVE, // 220
   IOCTLCMD_V2_CHECK_MEMORY,
   IOCTLCMD_V2_BLUE_SCREEN,
   IOCTLCMD_V2_ALLOW_CORE_DUMP,
   IOCTLCMD_V2_GET_STATS,
   IOCTLCMD_V2_SET_STATS, // 225
   IOCTLCMD_V2_SET_HARD_LIMIT,
   IOCTLCMD_V2_GET_HARD_LIMIT,
   IOCTLCMD_V2_GET_SET_MEM_INFO,
   IOCTLCMD_V2_GET_MEM_INFO,
   IOCTLCMD_V2_SET_MEM_INFO, // 230
   IOCTLCMD_V2_GET_VM_LIST,
   IOCTLCMD_V2_GET_VM_INFO,
   IOCTLCMD_V2_SET_VM_INFO,
   IOCTLCMD_V2_BROADCAST_IPI, // Linux SMP 2.2.8+ only
   IOCTLCMD_V2_FREE_RESOURCES,// Linux only, 235
   IOCTLCMD_V2_REDUCE_HARD_LIMIT, // Windows 2000 only
   IOCTLCMD_V2_LAST
} IOCTLCmdV2;

typedef enum IOCTLCmdV3 {
   IOCTLCMD_V3_NONE = 200,
   IOCTLCMD_V3_VERSION,
   IOCTLCMD_V3_CREATE_VM,
   IOCTLCMD_V3_BIND_VM,
   IOCTLCMD_V3_RELEASE_VM,
   IOCTLCMD_V3_GET_NUM_VMS,
   IOCTLCMD_V3_INIT_VM,
   IOCTLCMD_V3_LATE_INIT_VM,
   IOCTLCMD_V3_RUN_VM,
   IOCTLCMD_V3_LOOK_UP_MPN,
   IOCTLCMD_V3_LOCK_PAGE,
   IOCTLCMD_V3_UNLOCK_PAGE,
   IOCTLCMD_V3_APIC_BASE,
   IOCTLCMD_V3_IOAPIC_BASE,
   IOCTLCMD_V3_GET_STATS,
   IOCTLCMD_V3_SET_STATS,
   IOCTLCMD_V3_GET_HARD_LIMIT,
   IOCTLCMD_V3_SET_HARD_LIMIT,
   IOCTLCMD_V3_GET_MEM_INFO,
   IOCTLCMD_V3_SET_MEM_INFO,
   IOCTLCMD_V3_PAE_ENABLED,
   IOCTLCMD_V3_GET_TOTAL_MEM_USAGE,

   /*
    * Keep host-specific calls at the end so they can be undefined
    * without renumbering the common calls.
    */

   IOCTLCMD_V3_SET_UID,		// VMX86_DEVEL only
   IOCTLCMD_V3_IS_MP_SAFE,
   IOCTLCMD_V3_GET_MHZ_ESTIMATE,
   IOCTLCMD_V3_ALLOW_CORE_DUMP,

   IOCTLCMD_V3_CHECK_MEMORY,

   IOCTLCMD_V3_BROADCAST_IPI,	// SMP 2.2.8+ only

   IOCTLCMD_V3_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_V3_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_V3_FREE_PASSTHROUGH_IO,
   IOCTLCMD_V3_FREE_PASSTHROUGH_IRQ,
   IOCTLCMD_V3_START_PASSTHROUGH,
   IOCTLCMD_V3_STOP_PASSTHROUGH,
   IOCTLCMD_V3_QUERY_PASSTHROUGH,

   IOCTLCMD_V3_REGISTER_PERFCTR,
   IOCTLCMD_V3_START_PERFCTR,
   IOCTLCMD_V3_STOP_PERFCTR,
   IOCTLCMD_V3_RELEASE_PERFCTR,

   IOCTLCMD_V32_ALLOC_LOW_PAGES,
   IOCTLCMD_V32_FREE_LOW_PAGES,

   IOCTLCMD_V321_ADD_WORLD_ARG,
} IOCTLCmdV3;

typedef enum IOCTLCmdGSX2 {
   IOCTLCMD_GSX2_NONE = 200,
   IOCTLCMD_GSX2_VERSION,
   IOCTLCMD_GSX2_CREATE_VM,
   IOCTLCMD_GSX2_BIND_VM,
   IOCTLCMD_GSX2_RELEASE_VM,
   IOCTLCMD_GSX2_GET_NUM_VMS,
   IOCTLCMD_GSX2_INIT_VM,
   IOCTLCMD_GSX2_LATE_INIT_VM,
   IOCTLCMD_GSX2_RUN_VM,
   IOCTLCMD_GSX2_LOOK_UP_MPN,
   IOCTLCMD_GSX2_LOCK_PAGE,
   IOCTLCMD_GSX2_UNLOCK_PAGE,
   IOCTLCMD_GSX2_APIC_BASE,
   IOCTLCMD_GSX2_IOAPIC_BASE,
   IOCTLCMD_GSX2_GET_STATS,
   IOCTLCMD_GSX2_SET_STATS,
   IOCTLCMD_GSX2_GET_HARD_LIMIT,
   IOCTLCMD_GSX2_SET_HARD_LIMIT,
   IOCTLCMD_GSX2_GET_MEM_INFO,
   IOCTLCMD_GSX2_SET_MEM_INFO,
   IOCTLCMD_GSX2_PAE_ENABLED,
   IOCTLCMD_GSX2_GET_TOTAL_MEM_USAGE,
   IOCTLCMD_GSX2_SET_UID,
   IOCTLCMD_GSX2_IS_MP_SAFE,
   IOCTLCMD_GSX2_GET_MHZ_ESTIMATE,
   IOCTLCMD_GSX2_ALLOW_CORE_DUMP,
   IOCTLCMD_GSX2_CHECK_MEMORY,
   IOCTLCMD_GSX2_BROADCAST_IPI,
   IOCTLCMD_GSX2_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_GSX2_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_GSX2_FREE_PASSTHROUGH_IO,
   IOCTLCMD_GSX2_FREE_PASSTHROUGH_IRQ,
   IOCTLCMD_GSX2_START_PASSTHROUGH,
   IOCTLCMD_GSX2_STOP_PASSTHROUGH,
   IOCTLCMD_GSX2_QUERY_PASSTHROUGH,
   IOCTLCMD_GSX2_REGISTER_PERFCTR,
   IOCTLCMD_GSX2_START_PERFCTR,
   IOCTLCMD_GSX2_STOP_PERFCTR,
   IOCTLCMD_GSX2_RELEASE_PERFCTR,
   IOCTLCMD_GSX2_ALLOC_LOW_PAGES,
   IOCTLCMD_GSX2_FREE_LOW_PAGES,
   IOCTLCMD_GSX2_LAST
} IOCTLCmdGSX2;

typedef enum IOCTLCmdGSX25 {
   IOCTLCMD_GSX25_NONE = 200,
   IOCTLCMD_GSX25_VERSION,
   IOCTLCMD_GSX25_CREATE_VM,
   IOCTLCMD_GSX25_BIND_VM,
   IOCTLCMD_GSX25_RELEASE_VM,
   IOCTLCMD_GSX25_GET_NUM_VMS,
   IOCTLCMD_GSX25_INIT_VM,
   IOCTLCMD_GSX25_LATE_INIT_VM,
   IOCTLCMD_GSX25_RUN_VM,
   IOCTLCMD_GSX25_LOOK_UP_MPN,
   IOCTLCMD_GSX25_LOCK_PAGE,
   IOCTLCMD_GSX25_UNLOCK_PAGE,
   IOCTLCMD_GSX25_APIC_BASE,
   IOCTLCMD_GSX25_IOAPIC_BASE,
   IOCTLCMD_GSX25_GET_STATS,
   IOCTLCMD_GSX25_SET_STATS,
   IOCTLCMD_GSX25_GET_HARD_LIMIT,
   IOCTLCMD_GSX25_SET_HARD_LIMIT,
   IOCTLCMD_GSX25_GET_MEM_INFO,
   IOCTLCMD_GSX25_SET_MEM_INFO,
   IOCTLCMD_GSX25_PAE_ENABLED,
   IOCTLCMD_GSX25_GET_TOTAL_MEM_USAGE,
   IOCTLCMD_GSX25_SYNC_GET_TSCS,
   IOCTLCMD_GSX25_SYNC_SET_TSCS,
   IOCTLCMD_GSX25_GET_KHZ_ESTIMATE,
   IOCTLCMD_GSX25_SET_UID,		// VMX86_DEVEL only
   IOCTLCMD_GSX25_IS_MP_SAFE,
   IOCTLCMD_GSX25_ALLOW_CORE_DUMP,
   IOCTLCMD_GSX25_CHECK_MEMORY,
   IOCTLCMD_GSX25_BROADCAST_IPI,	// SMP 2.2.8+ only
   IOCTLCMD_GSX25_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_GSX25_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_GSX25_FREE_PASSTHROUGH_IO,
   IOCTLCMD_GSX25_FREE_PASSTHROUGH_IRQ,
   IOCTLCMD_GSX25_START_PASSTHROUGH,
   IOCTLCMD_GSX25_STOP_PASSTHROUGH,
   IOCTLCMD_GSX25_QUERY_PASSTHROUGH,
   IOCTLCMD_GSX25_REGISTER_PERFCTR,
   IOCTLCMD_GSX25_START_PERFCTR,
   IOCTLCMD_GSX25_STOP_PERFCTR,
   IOCTLCMD_GSX25_RELEASE_PERFCTR,
   IOCTLCMD_GSX25_ALLOC_LOW_PAGES,
   IOCTLCMD_GSX25_FREE_LOW_PAGES,
   IOCTLCMD_GSX25_LAST,
   IOCTLCMD_GSX251_GET_ALL_CPUID = IOCTLCMD_GSX25_LAST,
} IOCTLCmdGSX25;

typedef enum IOCTLCmdV4 {
   IOCTLCMD_V4_NONE = 200,
   IOCTLCMD_V4_VERSION,
   IOCTLCMD_V4_CREATE_VM,
   IOCTLCMD_V4_BIND_VM,
   IOCTLCMD_V4_RELEASE_VM,
   IOCTLCMD_V4_GET_NUM_VMS,
   IOCTLCMD_V4_INIT_VM,
   IOCTLCMD_V4_LATE_INIT_VM,
   IOCTLCMD_V4_RUN_VM,
   IOCTLCMD_V4_LOOK_UP_MPN,
   IOCTLCMD_V4_LOCK_PAGE,
   IOCTLCMD_V4_UNLOCK_PAGE,
   IOCTLCMD_V4_APIC_BASE,
   IOCTLCMD_V4_IOAPIC_BASE,
   IOCTLCMD_V4_GET_STATS,
   IOCTLCMD_V4_SET_STATS,
   IOCTLCMD_V4_GET_HARD_LIMIT,
   IOCTLCMD_V4_SET_HARD_LIMIT,
   IOCTLCMD_V4_GET_MEM_INFO,
   IOCTLCMD_V4_SET_MEM_INFO,
   IOCTLCMD_V4_PAE_ENABLED,
   IOCTLCMD_V4_GET_TOTAL_MEM_USAGE,
   IOCTLCMD_V4_ACK_USER_CALL,
   IOCTLCMD_V4_COMPLETE_USER_CALL,
   IOCTLCMD_V4_GET_KHZ_ESTIMATE,
   IOCTLCMD_V4_SET_UID,		// VMX86_DEVEL only
   IOCTLCMD_V4_IS_MP_SAFE,
   IOCTLCMD_V4_ALLOW_CORE_DUMP,
   IOCTLCMD_V4_CHECK_MEMORY,
   IOCTLCMD_V4_BROADCAST_IPI,	// SMP 2.2.8+ only
   IOCTLCMD_V4_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_V4_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_V4_FREE_PASSTHROUGH_IO,
   IOCTLCMD_V4_FREE_PASSTHROUGH_IRQ,
   IOCTLCMD_V4_START_PASSTHROUGH,
   IOCTLCMD_V4_STOP_PASSTHROUGH,
   IOCTLCMD_V4_QUERY_PASSTHROUGH,
   IOCTLCMD_V4_REGISTER_PERFCTR,
   IOCTLCMD_V4_START_PERFCTR,
   IOCTLCMD_V4_STOP_PERFCTR,
   IOCTLCMD_V4_RELEASE_PERFCTR,
   IOCTLCMD_V4_ALLOC_LOW_PAGES,
   IOCTLCMD_V4_FREE_LOW_PAGES,
   IOCTLCMD_V4_GET_SYSTEM_CLOCK_RATE,
   IOCTLCMD_V4_LAST
} IOCTLCmdV4;

typedef enum IOCTLCmdV45 {
   IOCTLCMD_V45_NONE = 2000,
   IOCTLCMD_V45_VERSION,
   IOCTLCMD_V45_CREATE_VM,
   IOCTLCMD_V45_BIND_VM,
   IOCTLCMD_V45_RELEASE_VM,
   IOCTLCMD_V45_GET_NUM_VMS,
   IOCTLCMD_V45_INIT_VM,
   IOCTLCMD_V45_LATE_INIT_VM,
   IOCTLCMD_V45_RUN_VM,
   IOCTLCMD_V45_LOOK_UP_MPN,
   IOCTLCMD_V45_LOCK_PAGE,
   IOCTLCMD_V45_UNLOCK_PAGE,
   IOCTLCMD_V45_APIC_BASE,
   IOCTLCMD_V45_IOAPIC_BASE,
   IOCTLCMD_V45_GET_HARD_LIMIT,
   IOCTLCMD_V45_SET_HARD_LIMIT,
   IOCTLCMD_V45_GET_MEM_INFO,
   IOCTLCMD_V45_SET_MEM_INFO,
   IOCTLCMD_V45_SET_MEM_USAGE,
   IOCTLCMD_V45_PAE_ENABLED,
   IOCTLCMD_V45_HOST_X86_64,
   IOCTLCMD_V45_GET_TOTAL_MEM_USAGE,
   IOCTLCMD_V45_ACK_USER_CALL,
   IOCTLCMD_V45_COMPLETE_USER_CALL,
   IOCTLCMD_V45_GET_KHZ_ESTIMATE,
   IOCTLCMD_V45_READ_PAGE,
   IOCTLCMD_V45_WRITE_PAGE,
   IOCTLCMD_V45_LOCK_PAGE_NEW,
   IOCTLCMD_V45_UNLOCK_PAGE_BY_MPN,
   IOCTLCMD_V45_MARK_LOCKEDVARANGE_CLEAN,
   IOCTLCMD_V45_ALLOC_LOCKED_PAGES,
   IOCTLCMD_V45_FREE_LOCKED_PAGES,
   IOCTLCMD_V45_GET_LOCKED_PAGES_LIST,
   IOCTLCMD_V45_MAP_LOCKED_PAGES,
   IOCTLCMD_V45_UNMAP_LOCKED_PAGES,
   IOCTLCMD_V45_SET_UID,	// VMX86_DEVEL only
   IOCTLCMD_V45_IS_MP_SAFE,
   IOCTLCMD_V45_ALLOW_CORE_DUMP,
   IOCTLCMD_V45_BROADCAST_IPI,	// SMP 2.2.8+ only
   IOCTLCMD_V45_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_V45_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_V45_FREE_PASSTHROUGH_IO,
   IOCTLCMD_V45_FREE_PASSTHROUGH_IRQ,
   IOCTLCMD_V45_START_PASSTHROUGH,
   IOCTLCMD_V45_STOP_PASSTHROUGH,
   IOCTLCMD_V45_QUERY_PASSTHROUGH,
   IOCTLCMD_V45_REGISTER_PERFCTR,
   IOCTLCMD_V45_START_PERFCTR,
   IOCTLCMD_V45_STOP_PERFCTR,
   IOCTLCMD_V45_RELEASE_PERFCTR,
   IOCTLCMD_V45_ALLOC_LOW_PAGES,
   IOCTLCMD_V45_FREE_LOW_PAGES,
   IOCTLCMD_V45_GET_ALL_CPUID,
   IOCTLCMD_V45_LAST
} IOCTLCmdV45;



#define VM_NAME_LENGTH        64
#define VM_USER_NAME_LENGTH   32

/*
 * Structure returned from GET_VM_LIST v2 ioctl.
 */
typedef struct {
   uint32   vmCount;
   uint32   vmUIDs[1];
} VMGetVMListResult;

typedef enum {
   VM_INFO_OFF,
   VM_INFO_RUNNING,
   VM_INFO_SUSPENDED,
   VM_INFO_BLOCKED,
} VMInfoState;

/*
 * Structure sent into VM_SET_INFO v2 ioctl.
 */
typedef struct {
   uint32		port;
   char			name[VM_NAME_LENGTH];
   char     		userName[VM_USER_NAME_LENGTH];
   VMInfoState 		state;
} VMSetVMInfoArgs;
/*
 * Structure returned from GET_VM_INFO v2 ioctl.
 */
typedef struct {
   uint32         vmUID;
   int32          port;
   char           name[VM_NAME_LENGTH];
   char           userName[VM_USER_NAME_LENGTH];
   VMInfoState	  state;
} VMGetVMInfoResult;

#ifdef VMX86_SERVER
#define FILE_LENGTH_ARGS_NAMELEN 256
/* Structure to get file length */
typedef struct VMFSFileLengthArgs {
   char fileName[FILE_LENGTH_ARGS_NAMELEN];
   long long length;
} VMFSFileLengthArgs;
#endif

typedef struct VMMemMgmtInfoCompat {
   int32   pageFaultRate;
   int32   lockFailRate;
   int32   maxLocked;
   int32   highWaterMark;
   int32   waitingForMemoryTime;
   int32   poweringOn;
   int32   hasFocus;
   uint32  guestmemSize;	// V45 only
} VMMemMgmtInfoCompat;

typedef struct VMMemMgmtInfoV4 {
   int32   pageFaultRate;
   int32   lockFailRate;
   int32   maxLocked;
   int32   currentLocked;
   int32   highWaterMark;
   int32   waitingForMemoryTime;
   int32   poweringOn;
   int32   hasFocus;
} VMMemMgmtInfoV4;

typedef struct VMMemMgmtInfoV45 {
   uint32          minAllocation;   // minimum pages for vm
   uint32          maxAllocation;   // maximum pages the vm could lock
   uint32          shares;          // proportional sharing weight
   uint32	   guestmemSize;
   uint32          nonpaged;        // overhead memory
   uint32          paged;           // nonmainMem guest memory
   uint32          locked;          // number of pages locked by this vm
   uint32          usedPct;         // % of guest memory being touched
   uint32          admitted;        // admission control
} VMMemMgmtInfoV45;

typedef struct VMSetMemInfoArgsV4 {
   VMMemMgmtInfoV4 info;
} VMSetMemInfoArgsV4;

typedef struct VMGetMemInfoArgsV4 {
   uint32           numVMs;
   uint32           numLockedPages;
   uint32           maxLockedPages;
   uint32           callerIndex;
   VMMemMgmtInfoV4  memMgmtInfo[1];
} VMGetMemInfoArgsV4;

#define VM_GET_MEM_INFO_SIZE_V4(numVMs) \
   ((numVMs - 1) * sizeof (VMMemMgmtInfoV4) + sizeof (VMGetMemInfoArgsV4))

typedef struct {
   uint32          numVMs;
   uint32          minVmMemPct;        // % of vm that must fit in maxLockedPages
   uint32          globalMinAllocation;// pages that must fit in maxLockedPages
   uint32          numLockedPages;     // total locked pages by all vms
   LockedPageLimit lockedPageLimit;    // set of locked page limits
   uint32          maxLockedPages;     // effective limit on locked pages
   uint32          callerIndex;        // this vm's index memInfo array
   VMMemMgmtInfoV45   memInfo[1];
} VMMemInfoArgsV45;

#define VM_GET_MEM_INFO_SIZE_V45(numVMs) \
   ((numVMs - 1) * sizeof (VMMemMgmtInfo) + sizeof (VMMemInfoArgsV45))

typedef struct {
   VA32     addr;
   unsigned len;
} VARangeV45;

typedef struct VMMPNList {
   uint32       mpnCount;
   VA32         mpn32List; /* list of 32 bit mpns */
} VMMPNList;

typedef struct VARange {
   VA32     addr;
   unsigned len;
   VA32     bv;
} VARange;

typedef struct VMMPNListVA {
   uint32       mpnCount;
   VA32         mpn32List;  /* list of 32 bit mpns */
   VA32		va;	    /* userspace VA. */
} VMMPNListVA;

typedef struct VMMUnlockPageByMPN {
   MPN32	  mpn;
   VA32 	  va;	    /* optional userspace VA */
} VMMUnlockPageByMPN;

typedef struct VMMReadWritePage {
   MPN32        mpn;
   VA32         buf;
} VMMReadWritePage;


struct passthrough_iorange {
  unsigned short ioBase;	/* Base of range to pass through */
  unsigned short numPorts;	/* Length of range */
};

typedef struct COWShareInfo {
   uint32       numPages;
   BPN          bpnList[PSHARE_BATCH_PAGES_MAX];
   MPN          mpnList[PSHARE_BATCH_PAGES_MAX];
   Bool         hintOnlyList[PSHARE_BATCH_PAGES_MAX];
   Bool         updateHints;
} COWShareInfo;

typedef struct COWHintInfo {
   uint32            nUpdates;
   PShare_HintUpdate updates[PSHARE_HINT_UPDATES_MAX];
} COWHintInfo;

typedef struct COWCheckInfo {
   uint32              numPages;
   PShare_COWCheckInfo check[PSHARE_MAX_COW_CHECK_PAGES];
} COWCheckInfo;

#ifdef linux

typedef struct {
   uint64              ioarg;	// 64bit address
   uint64              iocmd;	// 64bit command
} VMIoctl64;

/*
 * We can move it at the beginning of our ioctl list (just below
 * IOCTLCMD_VERSION). It just should not change very often...
 *
 * I'm allocating this number from matroxfb range which I maintain
 * and I will not reuse this number in any of my projects. --petr
 */
#define IOCTLCMD_IOCTL64  _IOW('n', 0xFF, VMIoctl64)

#endif

#endif // ifndef _IOCONTROLS_H_
