/* **********************************************************
 * Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
 * **********************************************************/

#ifndef _X86CPUID_H_
#define _X86CPUID_H_

/* http://www.sandpile.org/ia32/cpuid.htm */

#define INCLUDE_ALLOW_USERLEVEL
#define INCLUDE_ALLOW_VMX
#define INCLUDE_ALLOW_VMMEXT
#define INCLUDE_ALLOW_VMKERNEL
#define INCLUDE_ALLOW_MODULE
#define INCLUDE_ALLOW_DISTRIBUTE
#define INCLUDE_ALLOW_VMK_MODULE
#define INCLUDE_ALLOW_VMCORE
#include "includeCheck.h"

/*
 * CPUID Leaf 0x1, Register EDX (the original feature field)
 */

#define CPUID_FEATURE_COMMON_ID1EDX_FPU        0x00000001 /* 0 */
#define CPUID_FEATURE_COMMON_ID1EDX_VME        0x00000002 /* 1 */
#define CPUID_FEATURE_COMMON_ID1EDX_DBGE       0x00000004 /* 2 */
#define CPUID_FEATURE_COMMON_ID1EDX_PGSZE      0x00000008 /* 3 */
#define CPUID_FEATURE_COMMON_ID1EDX_TSC        0x00000010 /* 4 */
#define CPUID_FEATURE_COMMON_ID1EDX_MSR        0x00000020 /* 5 */
#define CPUID_FEATURE_COMMON_ID1EDX_PAE        0x00000040 /* 6 */ 
#define CPUID_FEATURE_COMMON_ID1EDX_MCK        0x00000080 /* 7 */ 
#define CPUID_FEATURE_COMMON_ID1EDX_CPMX       0x00000100 /* 8 */
#define CPUID_FEATURE_COMMON_ID1EDX_APIC       0x00000200 /* 9 */
#define CPUID_FEATURE_COMMON_ID1EDX_RSVD10     0x00000400 /* 10 */
#define CPUID_FEATURE_COMMON_ID1EDX_SEP        0x00000800 /* 11 */
#define CPUID_FEATURE_COMMON_ID1EDX_MTTR       0x00001000 /* 12 */
#define CPUID_FEATURE_COMMON_ID1EDX_PGE        0x00002000 /* 13 */
#define CPUID_FEATURE_COMMON_ID1EDX_MCA        0x00004000 /* 14 */
#define CPUID_FEATURE_COMMON_ID1EDX_CMOV       0x00008000 /* 15 */
#define CPUID_FEATURE_COMMON_ID1EDX_PAT        0x00010000 /* 16 */
#define CPUID_FEATURE_COMMON_ID1EDX_36PG       0x00020000 /* 17 */

#define CPUID_FEATURE_INTEL_ID1EDX_PSN         0x00040000 /* 18 */
#define CPUID_FEATURE_AMD_ID1EDX_RSVD18        0x00040000 /* 18 */

#define CPUID_FEATURE_COMMON_ID1EDX_CLFL       0x00080000 /* 19 */
#define CPUID_FEATURE_COMMON_ID1EDX_RSVD20     0x00100000 /* 20 */

#define CPUID_FEATURE_INTEL_ID1EDX_DTES        0x00200000 /* 21 */
#define CPUID_FEATURE_AMD_ID1EDX_RSVD21        0x00200000 /* 21 */

#define CPUID_FEATURE_INTEL_ID1EDX_ACPI        0x00400000 /* 22 */
#define CPUID_FEATURE_AMD_ID1EDX_RSVD22        0x00400000 /* 22 */

#define CPUID_FEATURE_COMMON_ID1EDX_MMX        0x00800000 /* 23 */
#define CPUID_FEATURE_COMMON_ID1EDX_FXSAVE     0x01000000 /* 24 */
#define CPUID_FEATURE_COMMON_ID1EDX_XMM        0x02000000 /* 25 */
#define CPUID_FEATURE_COMMON_ID1EDX_XMM2       0x04000000 /* 26 */

#define CPUID_FEATURE_INTEL_ID1EDX_SS          0x08000000 /* 27 */
#define CPUID_FEATURE_AMD_RSVD27               0x08000000 /* 27 */

#define CPUID_FEATURE_INTEL_ID1EDX_HT          0x10000000 /* 28 */
#define CPUID_FEATURE_AMD_ID1EDX_RSVD28        0x10000000 /* 28 */

#define CPUID_FEATURE_INTEL_ID1EDX_TM          0x20000000 /* 29 */
#define CPUID_FEATURE_AMD_ID1EDX_RSVD29        0x20000000 /* 29 */

#define CPUID_FEATURE_COMMON_ID1EDX_RSVD30     0x40000000 /* 30 */

#define CPUID_FEATURE_INTEL_ID1EDX_PBE         0x80000000 /* 31 */
#define CPUID_FEATURE_AMD_ID1EDX_RSVD31        0x80000000 /* 31 */

#define CPUID_FEATURE_INTEL_ID1EDX_RSVD		\
        (  CPUID_FEATURE_COMMON_ID1EDX_RSVD10	\
         | CPUID_FEATURE_COMMON_ID1EDX_RSVD20	\
	 | CPUID_FEATURE_COMMON_ID1EDX_RSVD30)

#define CPUID_FEATURE_AMD_ID1EDX_RSVD 		\
        (  CPUID_FEATURE_COMMON_ID1EDX_RSVD10 	\
         | CPUID_FEATURE_AMD_ID1EDX_RSVD18	\
         | CPUID_FEATURE_COMMON_ID1EDX_RSVD20	\
         | CPUID_FEATURE_AMD_ID1EDX_RSVD21	\
         | CPUID_FEATURE_AMD_ID1EDX_RSVD22      \
         | CPUID_FEATURE_AMD_ID1EDX_RSVD27	\
         | CPUID_FEATURE_AMD_ID1EDX_RSVD28      \
         | CPUID_FEATURE_AMD_ID1EDX_RSVD29      \
         | CPUID_FEATURE_AMD_ID1EDX_RSVD30      \
         | CPUID_FEATURE_AMD_ID1EDX_RSVD31)


/*
 * CPUID Leaf 0x1, Register ECX (Intel has started using this for features)
 */

#define CPUID_FEATURE_INTEL_ID1ECX_SSE3        0x00000001 /* 0 */

#define CPUID_FEATURE_INTEL_ID1ECX_RSVD1       0x00000002 /* 1 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD2       0x00000004 /* 2 */

#define CPUID_FEATURE_INTEL_ID1ECX_MWAIT       0x00000008 /* 3 */
#define CPUID_FEATURE_INTEL_ID1ECX_DSCPL       0x00000010 /* 4 */

#define CPUID_FEATURE_INTEL_ID1ECX_RSVD5       0x00000020 /* 5 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD6       0x00000040 /* 6 */

#define CPUID_FEATURE_INTEL_ID1ECX_EST         0x00000080 /* 7 */
#define CPUID_FEATURE_INTEL_ID1ECX_TM2         0x00000100 /* 8 */

#define CPUID_FEATURE_INTEL_ID1ECX_RSVD9       0x00000200 /* 9 */

#define CPUID_FEATURE_INTEL_ID1ECX_HTCACHE     0x00000400 /* 10 */

#define CPUID_FEATURE_INTEL_ID1ECX_RSVD11      0x00000800 /* 11 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD12      0x00001000 /* 12 */
#define CPUID_FEATURE_INTEL_ID1ECX_CMPX16      0x00002000 /* 13 */

#define CPUID_FEATURE_INTEL_ID1ECX_xPPR        0x00004000 /* 14 */

#define CPUID_FEATURE_INTEL_ID1ECX_RSVD15      0x00008000 /* 15 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD16      0x00010000 /* 16 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD17      0x00020000 /* 17 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD18      0x00040000 /* 18 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD19      0x00080000 /* 19 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD20      0x00100000 /* 20 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD21      0x00200000 /* 21 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD22      0x00400000 /* 22 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD23      0x00800000 /* 23 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD24      0x01000000 /* 24 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD25      0x02000000 /* 25 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD26      0x04000000 /* 26 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD27      0x08000000 /* 27 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD28      0x10000000 /* 28 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD29      0x20000000 /* 29 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD30      0x40000000 /* 30 */
#define CPUID_FEATURE_INTEL_ID1ECX_RSVD31      0x80000000 /* 31 */

#define CPUID_FEATURE_AMD_ID1ECX_RSVD          0xffffffff /* all */


/*
 * CPUID Leaf 0x80000001, Register EDX (AMD has always stored their
 * proprietary features here, AMD64 has forced Intel to start putting
 * feature bits here as well)
 */

#define CPUID_FEATURE_AMD_ID81EDX_FPU          0x00000001 /* 0 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD0      0x00000001 /* 0 */
#define CPUID_FEATURE_AMD_ID81EDX_VME          0x00000002 /* 1 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD1      0x00000002 /* 1 */
#define CPUID_FEATURE_AMD_ID81EDX_DBGE         0x00000004 /* 2 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD2      0x00000004 /* 2 */
#define CPUID_FEATURE_AMD_ID81EDX_PGSZE        0x00000008 /* 3 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD3      0x00000008 /* 3 */
#define CPUID_FEATURE_AMD_ID81EDX_TSC          0x00000010 /* 4 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD4      0x00000010 /* 4 */
#define CPUID_FEATURE_AMD_ID81EDX_MSR          0x00000020 /* 5 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD5      0x00000020 /* 5 */
#define CPUID_FEATURE_AMD_ID81EDX_PAE          0x00000040 /* 6 */ 
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD6      0x00000040 /* 6 */
#define CPUID_FEATURE_AMD_ID81EDX_MCK          0x00000080 /* 7 */ 
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD7      0x00000080 /* 7 */
#define CPUID_FEATURE_AMD_ID81EDX_CPMX         0x00000100 /* 8 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD8      0x00000100 /* 8 */
#define CPUID_FEATURE_AMD_ID81EDX_APIC         0x00000200 /* 9 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD9      0x00000200 /* 9 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD10    0x00000400 /* 10 */

#define CPUID_FEATURE_COMMON_ID81EDX_SYSC      0x00000800 /* 11 */

#define CPUID_FEATURE_AMD_ID81EDX_MTTR         0x00001000 /* 12 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD12     0x00001000 /* 12 */
#define CPUID_FEATURE_AMD_ID81EDX_PGE          0x00002000 /* 13 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD13     0x00002000 /* 13 */
#define CPUID_FEATURE_AMD_ID81EDX_MCA          0x00004000 /* 14 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD14     0x00004000 /* 14 */
#define CPUID_FEATURE_AMD_ID81EDX_CMOV         0x00008000 /* 15 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD15     0x00008000 /* 15 */
#define CPUID_FEATURE_AMD_ID81EDX_PAT          0x00010000 /* 16 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD16     0x00010000 /* 16 */
#define CPUID_FEATURE_AMD_ID81EDX_36PG         0x00020000 /* 17 */
#define CPUID_FEATURE_INTEL_ID81EDX_RSVD17     0x00020000 /* 17 */

#define CPUID_FEATURE_COMMON_ID81EDX_RSVD18    0x00040000 /* 18 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD19    0x00080000 /* 19 */

/* Intel spec doesn't say bit 20 is reserved - it'll most likely
   turn into NX soon. */
#define CPUID_FEATURE_AMD_ID81EDX_NX           0x00100000 /* 20 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD20    0x00100000 /* 20 */

#define CPUID_FEATURE_COMMON_ID81EDX_RSVD21    0x00200000 /* 21 */

#define CPUID_FEATURE_AMD_ID81EDX_MMXEXT       0x00400000 /* 22 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD22    0x00400000 /* 22 */
#define CPUID_FEATURE_AMD_ID81EDX_MMX          0x00800000 /* 23 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD23    0x00800000 /* 23 */
#define CPUID_FEATURE_AMD_ID81EDX_FXSAVE       0x01000000 /* 24 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD24    0x01000000 /* 24 */

#define CPUID_FEATURE_COMMON_ID81EDX_RSVD25    0x02000000 /* 25 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD26    0x04000000 /* 26 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD27    0x08000000 /* 27 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD28    0x10000000 /* 28 */

#define CPUID_FEATURE_COMMON_ID81EDX_LM        0x20000000 /* 29 */

#define CPUID_FEATURE_AMD_ID81EDX_3DNOWPLUS    0x40000000 /* 30 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD30    0x40000000 /* 30 */

#define CPUID_FEATURE_AMD_ID81EDX_3DNOW        0x80000000 /* 31 */
#define CPUID_FEATURE_COMMON_ID81EDX_RSVD31    0x80000000 /* 31 */


#define CPUID_PERMIT_LM() (Config_GetBool(FALSE, "monitor.enable64"))

/*
 * CPU checkpoint/migration compatibility masks. Bits set in these masks are
 * bits that must match in checkpoint/migration tests.
 */

#define CPT_DFLT_CPUID_ID1EDX_MASK      (~CPUID_FEATURE_COMMON_ID1EDX_APIC   &\
                                         ~CPUID_FEATURE_INTEL_ID1EDX_HT      &\
                                         ~CPUID_FEATURE_INTEL_ID1EDX_TM      &\
                                         ~CPUID_FEATURE_INTEL_ID1EDX_PBE)

#define CPT_DFLT_CPUID_ID1ECX_MASK      (~CPUID_FEATURE_INTEL_ID1ECX_TM2     &\
                                         ~CPUID_FEATURE_INTEL_ID1ECX_EST     &\
                                         ~CPUID_FEATURE_INTEL_ID1ECX_HTCACHE &\
                                         ~CPUID_FEATURE_INTEL_ID1ECX_xPPR)

#define CPT_DFLT_CPUID_ID81EDX_MASK                               \
   (~CPUID_FEATURE_AMD_ID81EDX_APIC                              &\
    ~(CPUID_PERMIT_LM() ? 0 : CPUID_FEATURE_COMMON_ID81EDX_LM)   &\
    ~CPUID_FEATURE_AMD_ID81EDX_NX)

/*
 * CPU features that are hidden from guests.
 */

#define CPUID_FEATURE_INTEL_ID1EDX_HIDDEN (CPUID_FEATURE_INTEL_ID1EDX_HT   |\
                                           CPUID_FEATURE_INTEL_ID1EDX_TM   |\
                                           CPUID_FEATURE_INTEL_ID1EDX_PBE)

#define CPUID_FEATURE_INTEL_ID1ECX_HIDDEN (CPUID_FEATURE_INTEL_ID1ECX_MWAIT |\
                                           CPUID_FEATURE_INTEL_ID1ECX_HTCACHE|\
                                           CPUID_FEATURE_INTEL_ID1ECX_xPPR)

#define CPUID_FEATURE_COMMON_ID81EDX_HIDDEN \
   ((CPUID_PERMIT_LM() ? 0 : CPUID_FEATURE_COMMON_ID81EDX_LM) |\
    CPUID_FEATURE_AMD_ID81EDX_NX)
                                             
/*
 * Other constants and macros.
 */

#define CPUID_STEPPING_MASK  0xf
#define CPUID_STEPPING(_eax) ((_eax) & CPUID_STEPPING_MASK)

#define CPUID_FAMILY(_eax)   (((_eax) >> 8) & 0xf)

/* Intel CPU Family */
#define CPUID_FAMILY_486     4
#define CPUID_FAMILY_P5      5
#define CPUID_FAMILY_P6      6
#define CPUID_FAMILY_EXTENDED 15

/* AMD CPU Family */
#define CPUID_FAMILY_5x86    4
#define CPUID_FAMILY_K5      5
#define CPUID_FAMILY_K6      5
#define CPUID_FAMILY_K7      6

#define CPUID_EXTENDED_FAMILY(_eax)   (((_eax) >> 20) & 0xff)

#define CPUID_EXTENDED_FAMILY_PENTIUM4  0
#define CPUID_EXTENDED_FAMILY_OPTERON   0

/*
 * Notice that CPUID families for Intel and AMD overlap. The following macros
 * should only be used AFTER the manufacturer has been established (through
 * the use of CPUID standard function 0).
 */
#define CPUID_FAMILY_IS_486(_eax) (CPUID_FAMILY(_eax) == CPUID_FAMILY_486)
#define CPUID_FAMILY_IS_P5(_eax)  (CPUID_FAMILY(_eax) == CPUID_FAMILY_P5)
#define CPUID_FAMILY_IS_P6(_eax)  (CPUID_FAMILY(_eax) == CPUID_FAMILY_P6)
#define CPUID_FAMILY_IS_PENTIUM4(_eax)  \
           (CPUID_FAMILY(_eax) == CPUID_FAMILY_EXTENDED && \
            CPUID_EXTENDED_FAMILY(_eax) == CPUID_EXTENDED_FAMILY_PENTIUM4)

#define CPUID_FAMILY_IS_OPTERON(_eax) \
           (CPUID_FAMILY(_eax) == CPUID_FAMILY_EXTENDED && \
            CPUID_EXTENDED_FAMILY(_eax) == CPUID_EXTENDED_FAMILY_OPTERON)

#define CPUID_MODEL(_eax)    (((_eax) >> 4) & 0xf)

#define CPUID_MODEL_PPRO       1
#define CPUID_MODEL_PII_3      3
#define CPUID_MODEL_PII_5      5
#define CPUID_MODEL_CELERON_6  6
#define CPUID_MODEL_PIII_7     7
#define CPUID_MODEL_PIII_8     8
#define CPUID_MODEL_PM         9
#define CPUID_MODEL_PIII_A     10
#define CPUID_MODEL_EXTENDED   15

#define CPUID_EXTENDED_MODEL(_eax)    (((_eax) >> 16) & 0xf)

#define CPUID_TYPE(_eax)    (((_eax) >> 12) & 0x3)
#define CPUID_TYPE_PRIMARY     0
#define CPUID_TYPE_OVERDRIVE   1
#define CPUID_TYPE_SECONDARY   2

#define CPUID_ID1EBX_LCPU_COUNT_MASK  0x00FF0000
#define CPUID_ID1EBX_LCPU_COUNT_SHIFT 16

#define CPUID_ID1EBX_APICID_MASK      0xFF000000
#define CPUID_ID1EBX_APICID_SHIFT     24

#endif
